package it.danieleverducci.ojo.ui;

import android.content.Context;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.navigation.fragment.NavHostFragment;

import org.videolan.libvlc.IVLCVout;
import org.videolan.libvlc.LibVLC;
import org.videolan.libvlc.Media;
import org.videolan.libvlc.MediaPlayer;

import java.util.ArrayList;

import it.danieleverducci.ojo.R;
import it.danieleverducci.ojo.databinding.FragmentSurveillanceBinding;
import it.danieleverducci.ojo.entities.Camera;

public class SurveillanceFragment extends Fragment implements MediaPlayer.EventListener, IVLCVout.Callback {

    private FragmentSurveillanceBinding binding;

    private CameraView cameraView;

    @Override
    public View onCreateView(
            LayoutInflater inflater, ViewGroup container,
            Bundle savedInstanceState
    ) {

        binding = FragmentSurveillanceBinding.inflate(inflater, container, false);
        return binding.getRoot();

    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

//        binding.buttonFirst.setOnClickListener(new View.OnClickListener() {
//            @Override
//            public void onClick(View view) {
//                NavHostFragment.findNavController(SurveillanceFragment.this)
//                        .navigate(R.id.action_FirstFragment_to_SecondFragment);
//            }
//        });

        // Start stream on surface view 1

        String rtspUrl = "rtsp://admin:Du4gdtPmXGCWT29@192.168.1.200:554/Streaming/Channels/101";
        Camera camera = new Camera("Camera 1", rtspUrl);

        ArrayList<String> options = new ArrayList<String>();
        options.add("--aout=opensles");
        options.add("--audio-time-stretch"); // time stretching
        options.add("-vvv"); // verbosity
        options.add("--aout=opensles");
        options.add("--avcodec-codec=h264");
        options.add("--file-logging");
        options.add("--logfile=vlc-log.txt");

        LibVLC libvlc = new LibVLC(getContext(), options);

        cameraView = new CameraView(
                getContext(),
                libvlc,
                camera,
                this
        );

        // Add to layout
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(1000, 500);
        binding.gridRowContainer.addView(cameraView.surfaceView, params);



    }

    @Override
    public void onStart() {
        super.onStart();

        cameraView.startPlayback();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
        // TODO Release player
    }

    /**
     * VLC media listener
     */
    @Override
    public void onEvent(MediaPlayer.Event event) {
        //TODO
    }

    /**
     * Video output callbacks
     */
    @Override
    public void onSurfacesCreated(IVLCVout vlcVout) {

    }

    @Override
    public void onSurfacesDestroyed(IVLCVout vlcVout) {

    }
    
    private void addCameraView(Camera camera) {
        
    }
    
    private class CameraView {
        protected SurfaceView surfaceView;
        protected MediaPlayer mediaPlayer;
        protected IVLCVout ivlcVout;
        protected Camera camera;
        protected LibVLC libvlc;

        public CameraView(Context context, LibVLC libvlc, Camera camera, IVLCVout.Callback callback) {
            this.camera = camera;
            this.libvlc = libvlc;

            surfaceView = new SurfaceView(context);
            SurfaceHolder holder = surfaceView.getHolder();

            holder.setKeepScreenOn(true);

            // Create media player
            mediaPlayer = new MediaPlayer(libvlc);
//            mediaPlayer.setEventListener(this);

            // Set up video output
            ivlcVout = mediaPlayer.getVLCVout();
            ivlcVout.setVideoView(surfaceView);
            ivlcVout.addCallback(callback);
            ivlcVout.attachViews();

            // Load media and start playing
            Media m = new Media(libvlc, Uri.parse(camera.getRtspUrl()));
            mediaPlayer.setMedia(m);

            // Register for view resize events
            final ViewTreeObserver observer= surfaceView.getViewTreeObserver();
            observer.addOnGlobalLayoutListener(() -> {
                // Set rendering size
                ivlcVout.setWindowSize(surfaceView.getWidth(), surfaceView.getHeight());
            });
        }

        /**
         * Starts the playback.
         * This must be called after the view has been laid out
         */
        public void startPlayback() {
            mediaPlayer.play();
        }
    }
}